from ya_business_api.core.mixins.asynchronous import AsyncAPIMixin
from ya_business_api.core.constants import CSRF_TOKEN_HEADER
from ya_business_api.companies.base_api import BaseCompaniesAPI
from ya_business_api.companies.dataclasses.companies import CompaniesResponse
from ya_business_api.companies.dataclasses.chain_list import ChainListResponse
from ya_business_api.companies.dataclasses.requests import CompaniesRequest, ChainListRequest
from ya_business_api.companies.parsers.chain_list_response import ChainListResponseParser

from typing import Optional, Union, Literal, overload
from time import monotonic
from logging import getLogger; log = getLogger(__name__)

from aiohttp.client import ClientSession


class AsyncCompaniesAPI(AsyncAPIMixin, BaseCompaniesAPI):
	chain_list_response_parser_csl = ChainListResponseParser

	def __init__(self, csrf_token: str, session: ClientSession) -> None:
		super().__init__(session, csrf_token)

		self.chain_list_response_parser = self.chain_list_response_parser_csl()

	@overload
	async def get_companies(self, request: Optional[CompaniesRequest] = None, *, raw: Literal[True]) -> dict: ...

	@overload
	async def get_companies(
		self,
		request: Optional[CompaniesRequest] = None,
		*,
		raw: Literal[False] = False,
	) -> CompaniesResponse: ...

	async def get_companies(
		self,
		request: Optional[CompaniesRequest] = None,
		*,
		raw: bool = False,
	) -> Union[CompaniesResponse, dict]:
		url = self.router.companies()
		request = request or CompaniesRequest()
		time_start = monotonic()

		async with self.session.get(url, allow_redirects=False, params=request.as_query_params()) as response:
			log.debug(f"A:COMPANIES[{response.status}] {monotonic() - time_start:.1f}s")
			self.check_response(response)

			if raw:
				return await response.json()

			return CompaniesResponse.model_validate_json(await response.text())

	@overload
	async def get_chain_list(self, request: ChainListRequest, *, raw: Literal[True]) -> str: ...

	@overload
	async def get_chain_list(self, request: ChainListRequest, *, raw: Literal[False] = False) -> ChainListResponse: ...

	async def get_chain_list(self, request: ChainListRequest, *, raw: bool = False) -> Union[ChainListResponse, str]:
		url = self.router.chain_list(request.tycoon_id)
		self.set_i_cookie()
		headers = {CSRF_TOKEN_HEADER: self.csrf_token}
		time_start = monotonic()

		async with self.session.post(
			url,
			params=request.as_query_params(),
			headers=headers,
			allow_redirects=False,
		) as response:
			log.debug(f"A:CHAIN_LIST[{response.status}] {monotonic() - time_start:.1f}s")
			self.check_response(response)
			content = await response.text()

			if raw:
				return content

			return self.chain_list_response_parser.parse(content)



from ya_business_api.core.router import Router

from abc import ABC, abstractmethod


class BaseAPI(ABC):
	router: Router

	def __init__(self) -> None:
		self.router = self.make_router()

	@abstractmethod
	def make_router(self) -> Router: ...



from ya_business_api.core.mixins.asynchronous import AsyncAPIMixin
from ya_business_api.core.constants import CSRF_TOKEN_HEADER
from ya_business_api.reviews.base_api import BaseReviewsAPI
from ya_business_api.reviews.constants import SUCCESS_ANSWER_RESPONSE
from ya_business_api.reviews.dataclasses.reviews import ReviewsResponse
from ya_business_api.reviews.dataclasses.requests import AnswerRequest, ReviewsRequest

from typing import Union, Literal, overload
from time import monotonic
from logging import getLogger; log = getLogger(__name__)

from aiohttp.client import ClientSession


class AsyncReviewsAPI(AsyncAPIMixin, BaseReviewsAPI):
	def __init__(self, csrf_token: str, session: ClientSession) -> None:
		super().__init__(session, csrf_token)

	@overload
	async def get_reviews(self, request: ReviewsRequest, *, raw: Literal[True]) -> dict: ...

	@overload
	async def get_reviews(self, request: ReviewsRequest, *, raw: Literal[False] = False) -> ReviewsResponse: ...

	async def get_reviews(self, request: ReviewsRequest, *, raw: bool = False) -> Union[ReviewsResponse, dict]:
		url = self.router.reviews(request.permanent_id)
		time_start = monotonic()

		async with self.session.get(url, params=request.as_query_params(), allow_redirects=False) as response:
			log.debug(f"A:REVIEWS[{response.status}] {monotonic() - time_start:.1f}s")
			self.check_response(response)

			if raw:
				return await response.json()

			return ReviewsResponse.model_validate_json(await response.text())

	async def send_answer(self, request: AnswerRequest) -> bool:
		url = self.router.answer()
		self.set_i_cookie()
		data = {
			"reviewId": request.review_id,
			"text": request.text,
			"answerCsrfToken": request.answer_csrf_token,
			"reviewsCsrfToken": request.reviews_csrf_token,
		}
		headers = {CSRF_TOKEN_HEADER: self.csrf_token}
		time_start = monotonic()

		async with self.session.post(url, json=data, headers=headers, allow_redirects=False) as response:
			log.debug(f"A:ANSWER[{response.status}] {monotonic() - time_start:.1f}s")
			self.check_response(response)
			result = await response.text() == SUCCESS_ANSWER_RESPONSE

		return result



from abc import ABC, abstractmethod
from typing import Collection, Union

from pydantic.main import BaseModel
from bs4.element import Tag


class BaseParser(ABC):
	@abstractmethod
	def parse(self, node_or_content: Union[Tag, str]) -> Union[Collection[BaseModel], BaseModel]: ...



from typing import Tuple, List, Dict, Union, Optional, Any

from pydantic.main import BaseModel
from pydantic.fields import Field


Number = Union[float, int]


class LocalizedValue(BaseModel):
	locale: str
	value: str


class Position(BaseModel):
	coordinates: Tuple[Number, Number]
	type: str		# Point


class AddressComponent(BaseModel):
	kind: str		# country, province, area, locality, street, house
	name: LocalizedValue


class AddressEntrance(BaseModel):
	normal_azimuth: Number
	pos: Position
	type: str		# main


class AddressTranslation(BaseModel):
	components: List[AddressComponent]
	formatted: LocalizedValue


class Address(BaseModel):
	formatted: LocalizedValue
	geo_id: int
	is_auto: bool
	translocal: str
	is_manual_one_line: bool

	# Optional fields
	components: List[AddressComponent] = []
	bounding_box: List[Tuple[Number, Number]] = []
	entrances: List[AddressEntrance] = []
	infos: List[LocalizedValue] = []
	translations: List[AddressTranslation] = []
	address_id: Optional[int] = None
	postal_code: Optional[str] = None
	precision: Optional[str] = None			# exact
	region_code: Optional[str] = None		# RU
	pos: Optional[Position] = None


class WorkInterval(BaseModel):
	day: str		# weekdays
	time_minutes_begin: int
	time_minutes_end: int


class GeoCampaign(BaseModel):
	has_active: bool = Field(alias="hasActive")
	has_draft: bool = Field(alias="hasDraft")


class Name(BaseModel):
	type: str		# main, synonym
	value: LocalizedValue


class PanoramaDirection(BaseModel):
	bearing: Number
	pitch: Number


class PanoramaSpan(BaseModel):
	horizontal: Number
	vertical: Number


class Panorama(BaseModel):
	direction: PanoramaDirection
	id: str
	pos: Position
	provider_id: int
	span: PanoramaSpan


class Phone(BaseModel):
	country_code: str
	region_code: str
	number: str
	formatted: str
	hide: bool
	type: str				# phone

	# Optional fields
	check_status: Optional[str] = None		# checked


class Rubric(BaseModel):
	features: List[Any]		# ???
	id: int
	is_main: bool = Field(alias="isMain")
	name: str


class ServiceProfile(BaseModel):
	external_path: str
	published: bool
	type: str		# maps, serp


class CompanyURL(BaseModel):
	hide: bool
	type: str									# main, social
	value: str

	# Optional fields
	social_login: Optional[str] = None
	social_network: Optional[str] = None		# vkontakte


class CompanyLogoAuthor(BaseModel):
	uid: int


class CompanyLogo(BaseModel):
	id: str
	tags: List[str]		# logo
	time_published: int
	url_template: str

	# Optional fields
	author: Optional[CompanyLogoAuthor] = None


class Chain(BaseModel):
	id: int
	permanent_id: int
	phones: List[Phone]
	urls: List[CompanyURL]
	display_name: str = Field(alias="displayName")
	names: List[Name]
	rubrics: List[Dict[str, Rubric]]
	brand_count: int = Field(alias="brandCount")


class Company(BaseModel):
	id: int
	tycoon_id: int
	permanent_id: int

	address: Address
	base_work_intervals: List[WorkInterval]
	display_name: str = Field(alias="displayName")
	emails: List[str]
	feature_values: List[Any]					# ???
	geo_campaign: GeoCampaign = Field(alias="geoCampaign")
	has_owner: bool
	is_top_rated: bool
	legal_info: dict							# ???
	nail: dict									# ???
	names: List[Name]
	no_access: bool = Field(alias="noAccess")
	object_role: str							# owner, delegate
	phones: List[Phone]
	photos: List[Any]							# ???
	price_lists: List[Any]						# ???
	profile: dict								# ???
	publishing_status: str						# publish
	rubrics: List[Dict[str, Rubric]]
	scheduled_work_intervals: List[Any]			# ???
	service_area: dict		# ???
	type: str									# ordinal, chain
	urls: List[CompanyURL]
	user_has_ydo_account: bool
	work_intervals: List[WorkInterval]

	# Optional fields
	diffs: Dict[str, int] = {}
	service_profiles: List[ServiceProfile] = []
	logo: Optional[CompanyLogo] = None
	from_geosearch: Optional[bool] = Field(None, alias="fromGeosearch")
	is_online: Optional[bool] = None
	owner: Optional[int] = None
	panorama: Optional[Panorama] = None
	rating: Optional[Number] = None
	reviews_count: Optional[int] = Field(None, alias="reviewsCount")


class CompaniesResponse(BaseModel):
	limit: int
	list_companies: List[Company] = Field(alias="listCompanies")
	page: int
	total: int



from ya_business_api.core.mixins.synchronous import SyncAPIMixin
from ya_business_api.service.base_api import BaseServiceAPI

from typing import Optional


class SyncServiceAPI(SyncAPIMixin, BaseServiceAPI):
	def get_csrf_token(self) -> Optional[str]:
		url = self.router.csrf_token()
		self.set_i_cookie()		# Server requires this cookie, but does not check its value.
		response = self.session.post(url, allow_redirects=False)

		if response.status_code == 488:
			return response.json().get(self.CSRF_TOKEN_FIELD)

		return None



from typing import Final, Set
from enum import Enum


BASE_URL: Final[str] = "https://yandex.ru/sprav"
PASSPORT_URL: Final[str] = "https://passport.yandex.ru"
INVALID_TOKEN_STATUSES: Final[Set[int]] = {488, 401}
CSRF_TOKEN_HEADER: Final[str] = "X-CSRF-Token"


class Cookie(Enum):
	SESSION_ID = "Session_id"
	SESSION_ID2 = "sessionid2"
	I = 'i'		# noqa: E741 - Yandex uses this name of cookie



from ya_business_api.core.exceptions import ParserError
from ya_business_api.companies.dataclasses.chain_list import CompanyCardWithPhoto
from ya_business_api.companies.parsers.company_card import CompanyCardParser

from json import loads

from bs4.element import Tag


class CompanyCardWithPhotoParser(CompanyCardParser):
	def parse(self, node: Tag) -> CompanyCardWithPhoto:
		company_card_with_photo_node = node.select_one('div.company-card-with-photo')

		if not company_card_with_photo_node:
			raise ParserError("Company card with photo doesn't exist")

		raw_data = company_card_with_photo_node.get("data-bem")

		if not isinstance(raw_data, str):
			raise ParserError("data-bem attribute doesn't exist")

		unvalidated_data = loads(raw_data)
		card_data = super().parse(node)

		return CompanyCardWithPhoto(**unvalidated_data.get("company-card-with-photo"), **card_data.model_dump())



from ya_business_api.reviews.async_api import AsyncReviewsAPI
from ya_business_api.companies.async_api import AsyncCompaniesAPI
from ya_business_api.service.async_api import AsyncServiceAPI
from ya_business_api.core.constants import Cookie
from ya_business_api.core.exceptions import CSRFTokenError

from typing import Optional
from logging import getLogger; log = getLogger(__name__)

from aiohttp.client import ClientSession


class AsyncAPI:
	reviews: AsyncReviewsAPI
	companies: AsyncCompaniesAPI
	session: ClientSession
	csrf_token: str

	def __init__(self, csrf_token: str, session: ClientSession) -> None:
		self.csrf_token = csrf_token
		self.session = session
		self.reviews = AsyncReviewsAPI(csrf_token, session)
		self.companies = AsyncCompaniesAPI(csrf_token, session)
		self.service = AsyncServiceAPI(session)

	@classmethod
	async def build(cls, session_id: str, session_id2: str, csrf_token: Optional[str] = None) -> "AsyncAPI":
		session = await cls.make_session(session_id, session_id2)

		if csrf_token is None:
			log.info("CSRF token was not specified. Attempting to receive a token automatically...")
			service_api = AsyncServiceAPI(session)
			csrf_token = await service_api.get_csrf_token()

			if csrf_token is None:
				raise CSRFTokenError("Failed to get CSRF token. It is not possible to create a client instance")

		return cls(csrf_token, session)

	@staticmethod
	async def make_session(session_id: str, session_id2: str) -> ClientSession:
		session = ClientSession()
		session.cookie_jar.update_cookies({
			Cookie.SESSION_ID.value: session_id,
			Cookie.SESSION_ID2.value: session_id2,
		})

		return session



from ya_business_api.reviews.sync_api import SyncReviewsAPI
from ya_business_api.companies.sync_api import SyncCompaniesAPI
from ya_business_api.service.sync_api import SyncServiceAPI
from ya_business_api.core.constants import Cookie
from ya_business_api.core.exceptions import CSRFTokenError

from typing import Optional
from logging import getLogger; log = getLogger(__name__)

from requests.sessions import Session


class SyncAPI:
	reviews: SyncReviewsAPI
	session: Session
	csrf_token: str

	def __init__(self, csrf_token: str, session: Session) -> None:
		self.csrf_token = csrf_token
		self.session = session
		self.reviews = SyncReviewsAPI(csrf_token, session)
		self.companies = SyncCompaniesAPI(csrf_token, session)
		self.service = SyncServiceAPI(session)

	@classmethod
	def build(cls, session_id: str, session_id2: str, csrf_token: Optional[str] = None) -> "SyncAPI":
		session = cls.make_session(session_id, session_id2)

		if csrf_token is None:
			log.info("CSRF token was not specified. Attempting to receive a token automatically...")
			service_api = SyncServiceAPI(session)
			csrf_token = service_api.get_csrf_token()

			if csrf_token is None:
				raise CSRFTokenError("Failed to get CSRF token. It is not possible to create a client instance")

		return cls(csrf_token, session)

	@staticmethod
	def make_session(session_id, session_id2) -> Session:
		session = Session()
		session.cookies.set(Cookie.SESSION_ID.value, session_id)
		session.cookies.set(Cookie.SESSION_ID2.value, session_id2)

		return session



from ya_business_api.core.base_api import BaseAPI
from ya_business_api.companies.router import CompaniesRouter


class BaseCompaniesAPI(BaseAPI):
	csrf_token: str
	router: CompaniesRouter

	def __init__(self, csrf_token: str) -> None:
		self.csrf_token = csrf_token

		super().__init__()

	def make_router(self) -> CompaniesRouter:
		return CompaniesRouter()



from ya_business_api.core.parser import BaseParser
from ya_business_api.core.exceptions import ParserError
from ya_business_api.companies.dataclasses.chain_list import ChainListResponse, CompanyCard
from ya_business_api.companies.parsers.pager import PagerParser
from ya_business_api.companies.parsers.chain_branches_list import ChainBranchesListParser
from ya_business_api.companies.parsers.company_card_safe_parser import CompanyCardSafeParser

from typing import Iterable, List

from bs4 import BeautifulSoup
from bs4.element import Tag


class ChainListResponseParser(BaseParser):
	chain_branches_list_parser_cls = ChainBranchesListParser
	company_card_parser_cls = CompanyCardSafeParser
	pager_parser_cls = PagerParser

	def __init__(self):
		self.pager_parser = self.pager_parser_cls()
		self.company_card_parser = self.company_card_parser_cls()
		self.chain_branches_list_parser = self.chain_branches_list_parser_cls()

	def parse(self, content: str) -> ChainListResponse:
		soup = BeautifulSoup(content, "html.parser")
		chain_branches_list_node = soup.select_one("div.chain-branches__list.i-bem")
		pager_node = soup.select_one("div.pager.i-bem")

		if not chain_branches_list_node:
			raise ParserError("Chain branches list node doesn't exist")

		if not pager_node:
			raise ParserError("Pager node doesn't exist")

		chain_branches_list = self.chain_branches_list_parser.parse(chain_branches_list_node)
		company_cards = self.parse_company_cards(soup.select("div.chain-branches__item"))
		pager = self.pager_parser.parse(pager_node)

		return ChainListResponse(chain_branches_list=chain_branches_list, company_cards=company_cards, pager=pager)

	def parse_company_cards(self, nodes: Iterable[Tag]) -> List[CompanyCard]:
		items = []

		for node in nodes:
			items.append(self.company_card_parser.parse(node))

		return items



from ya_business_api.core.router import Router
from ya_business_api.core.constants import BASE_URL


class CompaniesRouter(Router):
	def companies(self) -> str:
		return f"{BASE_URL}/api/companies"

	def chain_list(self, tycoon_id: int) -> str:
		return f"{BASE_URL}/api/view/chain/{tycoon_id}/list/"



from enum import Enum
from typing import Final


SUCCESS_ANSWER_RESPONSE: Final[str] = "OK"


class Ranking(Enum):
	BY_RATING_DESC = "by_rating_desc"
	BY_RATING_ASC = "by_rating_asc"
	BY_TIME = "by_time"



from ya_business_api.core.parser import BaseParser
from ya_business_api.companies.dataclasses.chain_list import CompanyCard
from ya_business_api.companies.parsers.company_card import CompanyCardParser
from ya_business_api.companies.parsers.company_card_with_photo import CompanyCardWithPhotoParser

from bs4.element import Tag


class CompanyCardSafeParser(BaseParser):
	"""
	Tries to parse company card with different parsers.
	"""
	company_card_with_photo_parser_cls = CompanyCardWithPhotoParser
	company_card_parser_cls = CompanyCardParser

	def __init__(self):
		self.company_card_with_photo_parser = self.company_card_with_photo_parser_cls()
		self.company_card_parser = self.company_card_parser_cls()

	def parse(self, node: Tag) -> CompanyCard:
		if node.select_one('div.company-card-with-photo.i-bem'):
			return self.company_card_with_photo_parser.parse(node)

		return self.company_card_parser.parse(node)



from ya_business_api.core.mixins.asynchronous import AsyncAPIMixin
from ya_business_api.service.base_api import BaseServiceAPI

from typing import Optional
from time import monotonic
from logging import getLogger; log = getLogger(__name__)


class AsyncServiceAPI(AsyncAPIMixin, BaseServiceAPI):
	async def get_csrf_token(self) -> Optional[str]:
		url = self.router.csrf_token()
		self.set_i_cookie()
		time_start = monotonic()

		async with self.session.post(url, allow_redirects=False) as response:
			log.debug(f"A:CSRF_TOKEN[{response.status}] {monotonic() - time_start:.1f}s")

			if response.status == 488:
				return (await response.json()).get(self.CSRF_TOKEN_FIELD)

		return None



from ya_business_api.core.router import Router
from ya_business_api.core.constants import BASE_URL


class ServiceRouter(Router):
	def csrf_token(self) -> str:
		return f"{BASE_URL}/api/view/chain/0/list/"



from ya_business_api.reviews.constants import Ranking

from typing import List, Optional

from pydantic.main import BaseModel
from pydantic.fields import Field


class Author(BaseModel):
	privacy: str
	user: str
	uid: int
	avatar: str


class InitChatData(BaseModel):
	entity_id: str = Field(alias="entityId")
	supplier_service_slug: str = Field(alias="supplierServiceSlug")
	name: str
	description: str
	entity_url: str = Field(alias="entityUrl")
	entity_image: str = Field(alias="entityImage")
	version: int


class OwnerComment(BaseModel):
	time_created: int
	text: str


class Review(BaseModel):
	id: str
	lang: str
	author: Author
	time_created: int
	snippet: str
	full_text: str
	rating: int
	cmnt_entity_id: str
	comments_count: int
	cmnt_official_token: str
	init_chat_data: InitChatData
	init_chat_token: str
	public_rating: bool
	business_answer_csrf_token: str

	# Optional fields
	owner_comment: Optional[OwnerComment] = None


class Pager(BaseModel):
	limit: int
	offset: int
	total: int


class Reviews(BaseModel):
	pager: Pager
	items: List[Review]
	csrf_token: str


class Filters(BaseModel):
	ranking: Ranking

	# Optional fields
	unread: Optional[str] = None


class CurrentState(BaseModel):
	filters: Filters


class ReviewsResponse(BaseModel):
	page: int
	current_state: CurrentState = Field(alias="currentState")
	list: Reviews



from ya_business_api.core.parser import BaseParser
from ya_business_api.core.exceptions import ParserError
from ya_business_api.companies.dataclasses.chain_list import Pager

from json import loads

from bs4.element import Tag


class PagerParser(BaseParser):
	def parse(self, node: Tag) -> Pager:
		raw_data = node.get("data-bem")

		if isinstance(raw_data, str):
			unvalidated_data = loads(raw_data)
			return Pager(**unvalidated_data.get("pager", {}))

		raise ParserError("data-bem attribute doesn't exist")



from typing import List

from pydantic.main import BaseModel
from pydantic.fields import Field


class ChainBranchesList(BaseModel):
	val: List[int]
	chain_id: int = Field(alias="chainId")
	geo_id: int = Field(alias="geoId")


class PagerBemJSONMods(BaseModel):
	data_source: str = Field(alias="data-source")
	ajax: bool


class PagerBemJSONPager(BaseModel):
	offset: int
	limit: int
	total: int


class PagerBemJSON(BaseModel):
	total_pages: int = Field(alias="totalPages")
	current_page: int = Field(alias="currentPage")
	params: dict
	url: str
	block: str
	mods: PagerBemJSONMods
	pager: PagerBemJSONPager
	path: str


class Pager(BaseModel):
	bemjson: PagerBemJSON


class CompanyCard(BaseModel):
	title: str
	address: str
	rubrics: str


class CompanyCardWithPhoto(CompanyCard):
	type: str		# ordinal
	company_id: int = Field(alias="companyId")
	permalink: int
	edit_photo_url: str = Field(alias="editPhotoUrl")
	url: str
	no_access: bool = Field(alias="noAccess")
	name: str


class ChainListResponse(BaseModel):
	chain_branches_list: ChainBranchesList
	company_cards: List[CompanyCard]
	pager: Pager



from ya_business_api.core.constants import INVALID_TOKEN_STATUSES, PASSPORT_URL, Cookie
from ya_business_api.core.exceptions import CSRFTokenError, AuthenticationError

from aiohttp.client import ClientSession, ClientResponse


class AsyncAPIMixin:
	session: ClientSession

	def __init__(self, session: ClientSession, *args, **kwargs) -> None:
		super().__init__(*args, **kwargs)

		self.session = session

	@staticmethod
	def check_response(response: ClientResponse) -> None:
		if response.status == 302 and response.headers.get('Location', '').startswith(PASSPORT_URL):
			raise AuthenticationError()

		if response.status in INVALID_TOKEN_STATUSES:
			raise CSRFTokenError()

		assert response.status == 200

	def set_i_cookie(self) -> None:
		"""
		Sets a stub value to the "i" cookie if it not specified.
		"""
		cookie_names = {cookie.key for cookie in self.session.cookie_jar}

		if Cookie.I.value not in cookie_names:
			self.session.cookie_jar.update_cookies({Cookie.I.value: ""})



from ya_business_api.core.constants import INVALID_TOKEN_STATUSES, PASSPORT_URL, Cookie
from ya_business_api.core.exceptions import CSRFTokenError, AuthenticationError

from requests.sessions import Session
from requests.models import Response


class SyncAPIMixin:
	session: Session

	def __init__(self, session: Session, *args, **kwargs) -> None:
		super().__init__(*args, **kwargs)

		self.session = session

	@staticmethod
	def check_response(response: Response) -> None:
		if response.status_code == 302 and getattr(response.next, 'url', "").startswith(PASSPORT_URL):
			raise AuthenticationError()

		if response.status_code in INVALID_TOKEN_STATUSES:
			raise CSRFTokenError()

		assert response.status_code == 200

	def set_i_cookie(self) -> None:
		"""
		Sets a stub value to the "i" cookie if it not specified.
		"""
		if Cookie.I.value not in self.session.cookies.keys():
			self.session.cookies.set(Cookie.I.value, "")



from typing import Optional

from pydantic.main import BaseModel


class CompaniesRequest(BaseModel):
	filter: Optional[str] = None
	page: Optional[int] = None

	def as_query_params(self) -> dict:
		result = {}

		if self.filter:
			result['filter'] = self.filter

		if self.page:
			result['page'] = self.page

		return result


class ChainListRequest(BaseModel):
	tycoon_id: int
	geo_id: Optional[int] = None
	page: Optional[int] = None

	def as_query_params(self) -> dict:
		result = {}

		if self.geo_id:
			result['geo_id'] = self.geo_id

		if self.page:
			result['page'] = self.page

		return result



from ya_business_api.reviews.constants import Ranking

from pydantic.main import BaseModel


class AnswerRequest(BaseModel):
	review_id: str
	text: str
	reviews_csrf_token: str
	answer_csrf_token: str


class ReviewsRequest(BaseModel):
	permanent_id: int
	ranking: Ranking = Ranking.BY_TIME
	unread: bool = False
	page: int = 1
	# aspectId: ???

	def as_query_params(self) -> dict:
		params = {"ranking": self.ranking.value, "page": self.page}

		if self.unread:
			params["unread"] = self.unread

		return params



from ya_business_api.core.mixins.synchronous import SyncAPIMixin
from ya_business_api.core.constants import CSRF_TOKEN_HEADER
from ya_business_api.companies.base_api import BaseCompaniesAPI
from ya_business_api.companies.dataclasses.companies import CompaniesResponse
from ya_business_api.companies.dataclasses.chain_list import ChainListResponse
from ya_business_api.companies.dataclasses.requests import CompaniesRequest, ChainListRequest
from ya_business_api.companies.parsers.chain_list_response import ChainListResponseParser

from typing import Optional, Union, Literal, overload

from requests.sessions import Session


class SyncCompaniesAPI(SyncAPIMixin, BaseCompaniesAPI):
	chain_list_response_parser_cls = ChainListResponseParser

	def __init__(self, csrf_token: str, session: Session) -> None:
		super().__init__(session, csrf_token)

		self.chain_list_response_parser = self.chain_list_response_parser_cls()

	@overload
	def get_companies(self, request: Optional[CompaniesRequest] = None, *, raw: Literal[True]) -> dict: ...

	@overload
	def get_companies(
		self,
		request: Optional[CompaniesRequest] = None,
		*,
		raw: Literal[False] = False,
	) -> CompaniesResponse: ...

	def get_companies(
		self,
		request: Optional[CompaniesRequest] = None,
		*,
		raw: bool = False,
	) -> Union[CompaniesResponse, dict]:
		url = self.router.companies()
		request = request or CompaniesRequest()
		response = self.session.get(url, allow_redirects=False, params=request.as_query_params())
		self.check_response(response)

		if raw:
			return response.json()

		return CompaniesResponse.model_validate_json(response.text)

	@overload
	def get_chain_list(self, request: ChainListRequest, *, raw: Literal[True]) -> str: ...

	@overload
	def get_chain_list(self, request: ChainListRequest, *, raw: Literal[False] = False) -> ChainListResponse: ...

	def get_chain_list(self, request: ChainListRequest, *, raw: bool = False) -> Union[ChainListResponse, str]:
		url = self.router.chain_list(request.tycoon_id)
		self.set_i_cookie()
		headers = {CSRF_TOKEN_HEADER: self.csrf_token}
		response = self.session.post(url, allow_redirects=False, params=request.as_query_params(), headers=headers)
		self.check_response(response)
		content = response.text

		if raw:
			return content

		return self.chain_list_response_parser.parse(content)



from ya_business_api.core.mixins.synchronous import SyncAPIMixin
from ya_business_api.core.constants import CSRF_TOKEN_HEADER
from ya_business_api.reviews.base_api import BaseReviewsAPI
from ya_business_api.reviews.constants import SUCCESS_ANSWER_RESPONSE
from ya_business_api.reviews.dataclasses.reviews import ReviewsResponse
from ya_business_api.reviews.dataclasses.requests import AnswerRequest, ReviewsRequest

from typing import Union, Literal, overload
from logging import getLogger; log = getLogger(__name__)

from requests.sessions import Session


class SyncReviewsAPI(SyncAPIMixin, BaseReviewsAPI):
	def __init__(self, csrf_token: str, session: Session) -> None:
		super().__init__(session, csrf_token)

	@overload
	def get_reviews(self, request: ReviewsRequest, *, raw: Literal[True]) -> dict: ...

	@overload
	def get_reviews(self, request: ReviewsRequest, *, raw: Literal[False] = False) -> ReviewsResponse: ...

	def get_reviews(self, request: ReviewsRequest, *, raw: bool = False) -> Union[ReviewsResponse, dict]:
		url = self.router.reviews(request.permanent_id)
		response = self.session.get(url, params=request.as_query_params(), allow_redirects=False)
		log.debug(f"REVIEWS[{response.status_code}] {response.elapsed.total_seconds()}s")
		self.check_response(response)

		if raw:
			return response.json()

		return ReviewsResponse.model_validate_json(response.text)

	def send_answer(self, request: AnswerRequest) -> bool:
		"""
		Sends an answer to review.

		Args:
			request: Request data to send.

		Returns:
			True - if answer has been sent successfully, otherwise - False.
		"""
		url = self.router.answer()
		self.set_i_cookie()		# Server requires this cookie, but does not check its value.
		response = self.session.post(
			url,
			json={
				"reviewId": request.review_id,
				"text": request.text,
				"answerCsrfToken": request.answer_csrf_token,
				"reviewsCsrfToken": request.reviews_csrf_token,
			},
			headers={
				CSRF_TOKEN_HEADER: self.csrf_token,
			},
			allow_redirects=False,
		)
		log.debug(f"ANSWER[{response.status_code}] {response.elapsed.total_seconds()}s")
		self.check_response(response)

		return response.text == SUCCESS_ANSWER_RESPONSE



class APIError(Exception):
	"""
	Basic API error.
	"""
	pass


class AuthenticationError(APIError):
	"""
	User authentication error.
	"""
	pass


class CSRFTokenError(APIError):
	"""
	Invalid CSRF token error.
	"""
	pass


class ParserError(APIError):
	"""
	Basic parser error.
	"""
	pass



from ya_business_api.core.parser import BaseParser
from ya_business_api.core.exceptions import ParserError
from ya_business_api.companies.dataclasses.chain_list import CompanyCard

from bs4.element import Tag


class CompanyCardParser(BaseParser):
	def parse(self, node: Tag) -> CompanyCard:
		title_node = node.select_one("div.company-card__title")
		address_node = node.select_one("div.company-card__address")
		rubrics_node = node.select_one("div.company-card__rubrics")

		if not title_node:
			raise ParserError("Title doesn't exist")

		if not address_node:
			raise ParserError("Address doesn't exist")

		if not rubrics_node:
			raise ParserError("Rubrics doesn't exist")

		title_link_node = title_node.select_one('a.link.link__control')

		if not title_link_node:
			raise ParserError("Link doesn't exist in the title node")

		return CompanyCard(
			title=title_link_node.text.strip(),
			address=address_node.text.strip(),
			rubrics=rubrics_node.text.strip(),
		)



from ya_business_api.core.router import Router
from ya_business_api.core.constants import BASE_URL


class ReviewsRouter(Router):
	def reviews(self, permanent_id: int) -> str:
		return f"{BASE_URL}/api/{permanent_id}/reviews"

	def answer(self) -> str:
		return f"{BASE_URL}/api/ugcpub/business-answer"



from ya_business_api.core.base_api import BaseAPI
from ya_business_api.service.router import ServiceRouter

from typing import Final


class BaseServiceAPI(BaseAPI):
	CSRF_TOKEN_FIELD: Final[str] = "csrf"

	router: ServiceRouter

	def make_router(self) -> ServiceRouter:
		return ServiceRouter()
